package com.buuz135.materialized.api.block;

import com.buuz135.materialized.Materialized;
import com.buuz135.materialized.api.material.BlockMaterial;
import com.buuz135.materialized.api.material.info.BlockPart;
import com.buuz135.materialized.proxy.client.RenderHelper;
import com.buuz135.materialized.utils.Reference;
import lombok.Setter;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.translation.I18n;
import net.minecraft.world.IBlockAccess;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.apache.commons.lang3.text.WordUtils;

import javax.annotation.Nonnull;
import java.util.Random;

public class MaterializedBlock extends Block {

    private BlockMaterial blockMaterial;
    private String name;
    @Setter
    private BlockPart.DropInfo dropInfo;

    public MaterializedBlock(BlockMaterial blockMaterial, String name, Material mat, int harvest, float hardness) {
        super(mat);
        this.name = name;
        this.blockMaterial = blockMaterial;
        this.setRegistryName(Reference.MODID, blockMaterial.getName() + name);
        this.func_149711_c(hardness);
        this.setHarvestLevel(blockMaterial.getTool(), harvest);
        this.func_149647_a(Materialized.creativeTab);
        this.func_149663_c(func_149732_F());
    }

    public void register() {
        GameRegistry.register(this);
        GameRegistry.register(new ItemBlock(this) {
            @Override
            public String func_77653_i(ItemStack stack) {
                return (I18n.func_94522_b("material." + name + ".name") ? I18n.func_74838_a("material." + name + ".name") : WordUtils.capitalize(name)) + " " + I18n.func_74838_a("type." + blockMaterial.getName() + ".name");
            }
        }, getRegistryName());
    }

    public void registerRender() {
        RenderHelper.registerBlockModel(this, blockMaterial);
    }

    @Override
    @Nonnull
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k() {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    public int func_180651_a(IBlockState state) {
        return dropInfo == null ? 0 : dropInfo.getMeta();
    }

    @Override
    public int func_149745_a(Random random) {
        return dropInfo == null || dropInfo.getAmount() == 1 ? 1 : 1 + random.nextInt(dropInfo.getAmount());
    }

    @Override
    public int func_149679_a(int fortune, Random random) {
        if (fortune > 0 && dropInfo != null && dropInfo.getAmount() > 1) {
            int i = random.nextInt(fortune * dropInfo.getFortuneModifier()) - 1;
            if (i < 0) {
                i = 0;
            }
            return this.func_149745_a(random) * (i + 1);
        } else {
            return this.func_149745_a(random);
        }
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune) {
        if (dropInfo.getItem() == null) return super.func_180660_a(state, rand, fortune);
        Item item = Item.func_111206_d(dropInfo.getItem());
        if (item != null) return item;
        item = Item.func_111206_d(dropInfo.getItem().replaceAll("this", Reference.MODID) + name);
        if (item != null) return item;
        return super.func_180660_a(state, rand, fortune);
    }

    @Override
    public int getExpDrop(IBlockState state, IBlockAccess world, BlockPos pos, int fortune) {
        return dropInfo.getXpDrop();
    }
}
